@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2018-2021, earthobservations developers.
+# Copyright (C) 2018-2021, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
 import json
 import logging
@@ -268,9 +268,9 @@ class ExportMixin:
             connection.execute(f"CREATE TABLE {tablename} AS SELECT * FROM origin;")  # noqa:S608
 
             weather_table = connection.table(tablename)
-            print(weather_table)  # noqa: T001
-            print("Cardinalities:")  # noqa: T001
-            print(weather_table.to_df().count())  # noqa: T001
+            print(weather_table)  # noqa: T201
+            print("Cardinalities:")  # noqa: T201
+            print(weather_table.to_df().count())  # noqa: T201
             connection.close()
             log.info("Writing to DuckDB finished")
 
@@ -517,7 +517,10 @@ def convert_datetimes(df: pd.DataFrame) -> pd.DataFrame:
     df: pd.DataFrame = df.copy(deep=True)
 
     date_columns = list(df.select_dtypes(include=[pd.DatetimeTZDtype]).columns)
+    date_columns.extend([Columns.FROM_DATE.value, Columns.TO_DATE.value, Columns.DATE.value])
+    date_columns = set(date_columns)
     for date_column in date_columns:
-        df[date_column] = df[date_column].apply(lambda d: d.isoformat() if pd.notna(d) else None)
+        if date_column in df:
+            df[date_column] = df[date_column].apply(lambda d: d.isoformat() if pd.notna(d) else None)
 
     return df
