@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2018-2021, earthobservations developers.
+# Copyright (C) 2018-2021, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
 import logging
 from abc import abstractmethod
@@ -18,7 +18,11 @@ from rapidfuzz import fuzz, process
 
 from wetterdienst.core.core import Core
 from wetterdienst.core.scalar.result import StationsResult
-from wetterdienst.exceptions import InvalidEnumeration, StartDateEndDateError
+from wetterdienst.exceptions import (
+    InvalidEnumeration,
+    NoParametersFound,
+    StartDateEndDateError,
+)
 from wetterdienst.metadata.columns import Columns
 from wetterdienst.metadata.datarange import DataRange
 from wetterdienst.metadata.kind import Kind
@@ -35,7 +39,7 @@ EARTH_RADIUS_KM = 6371
 
 
 class ScalarRequestCore(Core):
-    """Core for stations information of a source"""
+    """Core for stations_result information of a source"""
 
     @property
     @abstractmethod
@@ -65,8 +69,19 @@ class ScalarRequestCore(Core):
     def frequency(self) -> Frequency:
         """Frequency for the given resolution, used to create a full date range for
         mering"""
+        if self.resolution == Resolution.DYNAMIC:
+            return self._dynamic_frequency
         return Frequency[self.resolution.name]
 
+    @property
+    def dynamic_frequency(self) -> Optional[Frequency]:
+        return self._dynamic_frequency
+
+    @dynamic_frequency.setter
+    def dynamic_frequency(self, df) -> None:
+        if df:
+            self._dynamic_frequency = parse_enumeration_from_template(df, Frequency)
+
     @property
     @abstractmethod
     def _period_type(self) -> PeriodType:
@@ -80,16 +95,11 @@ class ScalarRequestCore(Core):
         pass
 
     @property
+    @abstractmethod
     def _parameter_base(self) -> Enum:
         """parameter base enumeration from which parameters can be parsed e.g.
         DWDObservationParameter"""
-        if self._has_datasets:
-            if not self._unique_dataset:
-                raise NotImplementedError(
-                    "implement _parameter_base enumeration that has "
-                    "all parameters of one resolution stored in one place"
-                )
-        return
+        pass
 
     @property
     @abstractmethod
@@ -111,13 +121,7 @@ class ScalarRequestCore(Core):
         if self._has_datasets:
             raise NotImplementedError("implement _dataset_base enumeration that contains available datasets")
 
-    @property
-    def _dataset_tree(self) -> Optional[object]:
-        """Detailed dataset tree with all parameters per dataset"""
-        if self._has_datasets:
-            raise NotImplementedError(
-                "implement _dataset_tree class that contains available datasets " "and their parameters"
-            )
+        return self._resolution_base
 
     @property
     def _unique_dataset(self) -> bool:
@@ -165,7 +169,7 @@ class ScalarRequestCore(Core):
         """Class to get the values for a request"""
         pass
 
-    # Columns that should be contained within any stations information
+    # Columns that should be contained within any stations_result information
     _base_columns = (
         Columns.STATION_ID.value,
         Columns.FROM_DATE.value,
@@ -176,17 +180,6 @@ class ScalarRequestCore(Core):
         Columns.NAME.value,
         Columns.STATE.value,
     )
-    # TODO: eventually this can be matched with the type coercion of station data to get
-    #  similar types of floats and strings
-    # Dtype mapping for stations
-    _dtype_mapping = {
-        Columns.STATION_ID.value: str,
-        Columns.HEIGHT.value: float,
-        Columns.LATITUDE.value: float,
-        Columns.LONGITUDE.value: float,
-        Columns.NAME.value: str,
-        Columns.STATE.value: str,
-    }
 
     def _parse_period(self, period: Period) -> Optional[List[Period]]:
         """
@@ -225,7 +218,7 @@ class ScalarRequestCore(Core):
 
         parameters = []
 
-        for parameter in pd.Series(parameter):
+        for par in pd.Series(parameter):
 
             # Each parameter can either be
             #  - a dataset : gets all data from the dataset
@@ -234,49 +227,96 @@ class ScalarRequestCore(Core):
             #  - a tuple of parameter -> dataset : to decide from which dataset
             #    the parameter is taken
             try:
-                parameter, dataset = pd.Series(parameter)
+                parameter, dataset = pd.Series(par)
             except (ValueError, TypeError):
-                parameter, dataset = parameter, parameter
+                parameter, dataset = par, par
+
+            try:
+                parameter = parameter.name
+            except AttributeError:
+                pass
+
+            try:
+                dataset = dataset.name
+            except AttributeError:
+                pass
 
             # Prefix return values
-            parameter_, dataset_ = None, None
+            parameter_, dataset_ = self._parse_dataset_and_parameter(parameter, dataset)
 
-            # Try to parse dataset
-            try:
-                dataset_ = parse_enumeration_from_template(dataset, self._dataset_base)
-            except InvalidEnumeration:
-                pass
-
-            if parameter == dataset and dataset_:
-                parameters.append((dataset_, dataset_))
-                continue
-
-            try:
-                # First parse parameter
-                parameter_ = parse_enumeration_from_template(parameter, self._parameter_base[self._dataset_accessor])
-            except (InvalidEnumeration, TypeError):
-                pass
-            else:
-                if self._unique_dataset:
-                    # If unique dataset the dataset is given by the accessor
-                    # and the parameter is not a subset of a dataset
-                    dataset_ = self._dataset_base[self._dataset_accessor]
-                elif not dataset_:
-                    # If there's multiple datasets the mapping defines which one
-                    # is taken for the given parameter
-                    dataset_ = self._parameter_to_dataset_mapping[self.resolution][parameter_]
-
-                if not self._unique_dataset:
-                    # Parameter then has to be taken from the datasets definition
-                    parameter_ = self._dataset_tree[self._dataset_accessor][dataset_.name][parameter_.name]
+            if not (parameter_ and dataset_):
+                parameter_, dataset_ = self._parse_parameter_and_dataset(parameter)
 
+            if parameter_ and dataset_:
                 parameters.append((parameter_, dataset_))
-
-            if not parameter_:
+            else:
                 log.info(f"parameter {parameter} could not be parsed from ({enums})")
 
         return parameters
 
+    def _parse_dataset_and_parameter(self, parameter, dataset) -> Tuple[Optional[Enum], Optional[Enum]]:
+        """
+        Parse parameters for cases like
+            - parameter=("climate_summary", ) or
+            - parameter=(("precipitation_height", "climate_summary"))
+        :param self:
+        :param parameter:
+        :param dataset:
+        :return:
+        """
+        parameter_, dataset_ = None, None
+
+        try:
+            dataset_ = parse_enumeration_from_template(dataset, self._dataset_base)
+        except InvalidEnumeration:
+            pass
+
+        if dataset_ and self._has_datasets and not self._unique_dataset:
+            try:
+                self._parameter_base[self._dataset_accessor][dataset_.name]
+            except (KeyError, AttributeError):
+                log.warning(f"dataset {dataset_.name} is not a valid dataset for resolution {self._dataset_accessor}")
+                return None, None
+
+        if dataset_:
+            if parameter == dataset:
+                # Case 1: entire dataset e.g. parameter="climate_summary"
+                parameter_, dataset_ = dataset_, dataset_
+            else:
+                # Case 2: dataset and parameter e.g. (precipitation_height, climate_summary)
+                try:
+                    parameter_ = parse_enumeration_from_template(
+                        parameter, self._parameter_base[self._dataset_accessor][dataset_.name]
+                    )
+                except (InvalidEnumeration, TypeError):
+                    pass
+
+        return parameter_, dataset_
+
+    def _parse_parameter_and_dataset(self, parameter) -> Tuple[Enum, Enum]:
+        """Try to parse dataset first e.g. when "climate_summary" or
+        "precipitation_height", "climate_summary" is requested
+
+        :param parameter:
+        :return:
+        """
+
+        parameter_, dataset_ = None, None
+
+        flat_parameters = {par for par in self._parameter_base[self._dataset_accessor] if hasattr(par, "name")}
+
+        for par in flat_parameters:
+            if par.name.lower() == parameter.lower() or par.value.lower() == parameter.lower():
+                parameter_ = par
+                break
+
+        if parameter_:
+            dataset_name = parameter_.__class__.__name__
+
+            dataset_ = parse_enumeration_from_template(dataset_name, self._dataset_base)
+
+        return parameter_, dataset_
+
     @staticmethod
     def _parse_station_id(series: pd.Series) -> pd.Series:
         """
@@ -288,18 +328,6 @@ class ScalarRequestCore(Core):
         """
         return series.astype(str)
 
-    def __eq__(self, other) -> bool:
-        """Equal method of request object"""
-        return (
-            self.parameter == other.parameter
-            and self.resolution == other.resolution
-            and self.period == other.period
-            and self.start_date == other.start_date
-            and self.end_date == other.end_date
-            and self.humanize == other.humanize
-            and self.tidy == other.tidy
-        )
-
     def __init__(
         self,
         parameter: Tuple[Union[str, Enum]],
@@ -313,13 +341,10 @@ class ScalarRequestCore(Core):
         :param parameter: requested parameter(s)
         :param resolution: requested resolution
         :param period: requested period(s)
-        :param start_date: Start date for filtering stations for their available data
-        :param end_date:   End date for filtering stations for their available data
-        :param humanize: boolean if parameters should be humanized
-        :param tidy: boolean if data should be tidied
-        :param si_units: boolean if values should be converted to si units
+        :param start_date: Start date for filtering stations_result for their available data
+        :param end_date:   End date for filtering stations_result for their available data
         """
-
+        settings = copy(Settings)
         super().__init__()
 
         self.resolution = parse_enumeration_from_template(resolution, self._resolution_base, Resolution)
@@ -328,26 +353,60 @@ class ScalarRequestCore(Core):
         self.start_date, self.end_date = self.convert_timestamps(start_date, end_date)
         self.parameter = self._parse_parameter(parameter)
 
-        self.humanize = copy(Settings.humanize)
+        if not self.parameter:
+            raise NoParametersFound("no valid parameters could be parsed from given argument")
 
-        tidy = copy(Settings.tidy)
+        self.humanize = settings.humanize
+
+        tidy = settings.tidy
         if self._has_datasets:
             tidy = tidy or any([parameter not in self._dataset_base for parameter, dataset in self.parameter])
         self.tidy = tidy
 
-        self.si_units = copy(Settings.si_units)
+        self.si_units = settings.si_units
 
-        log.info(
-            f"Processing request for "
-            f"provider={self.provider}, "
-            f"parameter={self.parameter}, "
-            f"resolution={self.resolution}, "
-            f"period={self.period}, "
-            f"start_date={self.start_date}, "
-            f"end_date={self.end_date}, "
+        # skip empty stations
+        self.skip_empty = self.tidy and settings.skip_empty
+        self.skip_threshold = settings.skip_threshold
+        self.dropna = self.tidy and settings.dropna
+
+        if not tidy and settings.skip_empty:
+            log.warning("option 'skip_empty' is only available with option 'tidy' and is thus ignored in this request.")
+
+        if not tidy and settings.dropna:
+            log.warning("option 'dropna' is only available with option 'tidy' and is thus ignored in this request.")
+
+        # optional attribute for dynamic resolutions
+        if self.resolution == Resolution.DYNAMIC:
+            self._dynamic_frequency = None
+
+        log.info(f"Processing request {self.__repr__()}")
+
+    def __repr__(self):
+        """Representation of request object"""
+        parameters_joined = ", ".join([f"({parameter.value}/{dataset.value})" for parameter, dataset in self.parameter])
+        periods_joined = self.period and ", ".join([period.value for period in self.period])
+
+        return (
+            f"{self.__class__.__name__}("
+            f"parameter=[{parameters_joined}], "
+            f"resolution={self.resolution.value}, "
+            f"period=[{periods_joined}], "
+            f"start_date={str(self.start_date)}, "
+            f"end_date={str(self.end_date)}, "
             f"humanize={self.humanize}, "
             f"tidy={self.tidy}, "
-            f"si_units={self.si_units}"
+            f"si_units={self.si_units})"
+        )
+
+    def __eq__(self, other) -> bool:
+        """Equal method of request object"""
+        return (
+            self.parameter == other.parameter
+            and self.resolution == other.resolution
+            and self.period == other.period
+            and self.start_date == other.start_date
+            and self.end_date == other.end_date
         )
 
     @staticmethod
@@ -359,8 +418,8 @@ class ScalarRequestCore(Core):
         Sort out start_date vs. end_date, parse strings to datetime
         objects and finally convert both to pd.Timestamp types.
 
-        :param start_date: Start date for filtering stations for their available data
-        :param end_date:   End date for filtering stations for their available data
+        :param start_date: Start date for filtering stations_result for their available data
+        :param end_date:   End date for filtering stations_result for their available data
         :return:           pd.Timestamp objects tuple of (start_date, end_date)
         """
 
@@ -434,15 +493,19 @@ class ScalarRequestCore(Core):
 
             for f in filter_:
                 parameters[f.name.lower()] = {}
+
                 for parameter in cls._parameter_base[f.name]:
+                    if not hasattr(parameter, "name"):
+                        continue
+
                     parameters[f.name.lower()][parameter.name.lower()] = {}
 
                     if cls._unique_dataset:
                         origin_unit, si_unit = cls._unit_tree[f.name][parameter.name].value
                     else:
-                        dataset = cls._parameter_to_dataset_mapping[f][parameter]
-
-                        origin_unit, si_unit = cls._unit_tree[f.name][dataset.name][parameter.name].value
+                        origin_unit, si_unit = cls._unit_tree[f.name][parameter.__class__.__name__][
+                            parameter.name
+                        ].value
 
                     parameters[f.name.lower()][parameter.name.lower()]["origin"] = cls._format_unit(origin_unit)
 
@@ -462,25 +525,26 @@ class ScalarRequestCore(Core):
         for f in filter_:
             parameters[f.name.lower()] = {}
 
-            for dataset in cls._dataset_tree[f.name].__dict__:
-                if dataset.startswith("_") or dataset not in datasets_filter:
+            for dataset in cls._parameter_base[f.name]:
+                if hasattr(dataset, "name"):
                     continue
 
-                parameters[f.name.lower()][dataset.lower()] = {}
+                dataset_name = dataset.__name__.lower()
+                if dataset_name.startswith("_") or dataset_name.upper() not in datasets_filter:
+                    continue
 
-                for parameter in cls._dataset_tree[f.name][dataset]:
+                parameters[f.name.lower()][dataset_name] = {}
 
-                    parameters[f.name.lower()][dataset.lower()][parameter.name.lower()] = {}
+                for parameter in dataset:
+                    parameters[f.name.lower()][dataset_name][parameter.name.lower()] = {}
 
-                    origin_unit, si_unit = cls._unit_tree[f.name][dataset][parameter.name].value
+                    origin_unit, si_unit = cls._unit_tree[f.name][dataset_name.upper()][parameter.name].value
 
-                    parameters[f.name.lower()][dataset.lower()][parameter.name.lower()]["origin"] = cls._format_unit(
+                    parameters[f.name.lower()][dataset_name][parameter.name.lower()]["origin"] = cls._format_unit(
                         origin_unit
                     )
 
-                    parameters[f.name.lower()][dataset.lower()][parameter.name.lower()]["si"] = cls._format_unit(
-                        si_unit
-                    )
+                    parameters[f.name.lower()][dataset_name][parameter.name.lower()]["si"] = cls._format_unit(si_unit)
 
         return parameters
 
@@ -501,14 +565,20 @@ class ScalarRequestCore(Core):
             pd.Series(filter_).apply(parse_enumeration_from_template, args=(cls._resolution_base, Resolution)).tolist()
         )
 
-    def _coerce_meta_fields(self, df) -> pd.DataFrame:
+    @staticmethod
+    def _coerce_meta_fields(df) -> pd.DataFrame:
         """
         Method for metadata column coercion.
 
         :param df: DataFrame with columns as strings
         :return: DataFrame with columns coerced to date etc.
         """
-        df = df.astype(self._dtype_mapping)
+        df[Columns.STATION_ID.value] = pd.Series(df[Columns.STATION_ID.value].values, dtype=str)
+        df[Columns.HEIGHT.value] = pd.Series(df[Columns.HEIGHT.value], dtype=float)
+        df[Columns.LATITUDE.value] = df[Columns.LATITUDE.value].astype(float)
+        df[Columns.LONGITUDE.value] = df[Columns.LONGITUDE.value].astype(float)
+        df[Columns.NAME.value] = pd.Series(df[Columns.NAME.value].values, dtype=str)
+        df[Columns.STATE.value] = pd.Series(df[Columns.STATE.value].values, dtype=str)
 
         df[Columns.FROM_DATE.value] = pd.to_datetime(
             df[Columns.FROM_DATE.value], infer_datetime_format=True
@@ -533,22 +603,22 @@ class ScalarRequestCore(Core):
         """
         Wraps the _all method and applies date filters.
 
-        :return: pandas.DataFrame with the information of different available stations
+        :return: pandas.DataFrame with the information of different available stations_result
         """
-        df = self._all()
+        df = self._all().copy().reset_index(drop=True)
 
         df = df.reindex(columns=self._base_columns)
 
         df = self._coerce_meta_fields(df)
 
-        return StationsResult(self, df.copy().reset_index(drop=True))
+        return StationsResult(self, df.reset_index(drop=True))
 
     def filter_by_station_id(self, station_id: Tuple[str, ...]) -> StationsResult:
         """
-        Method to filter stations by station ids
+        Method to filter stations_result by station ids
 
-        :param station_id: list of stations that are requested
-        :return: df with filtered stations
+        :param station_id: list of stations_result that are requested
+        :return: df with filtered stations_result
         """
         df = self.all().df
 
@@ -562,7 +632,7 @@ class ScalarRequestCore(Core):
 
     def filter_by_name(self, name: str, first: bool = True, threshold: int = 90) -> StationsResult:
         """
-        Method to filter stations for station name using string comparison.
+        Method to filter stations_result for station name using string comparison.
 
         :param name: name of looked up station
         :param first: boolean if only first station is returned
@@ -609,12 +679,12 @@ class ScalarRequestCore(Core):
     ) -> StationsResult:
         """
         Wrapper for get_nearby_stations_by_number using the given parameter set. Returns
-        nearest stations defined by number.
+        nearest stations_result defined by number.
 
         :param latitude: latitude in degrees
         :param longitude: longitude in degrees
-        :param rank: number of stations to be returned, greater 0
-        :return: pandas.DataFrame with station information for the selected stations
+        :param rank: number of stations_result to be returned, greater 0
+        :return: pandas.DataFrame with station information for the selected stations_result
         """
         rank = int(rank)
 
@@ -634,11 +704,12 @@ class ScalarRequestCore(Core):
 
         df = df.iloc[indices_nearest_neighbours.flatten(), :].reset_index(drop=True)
 
-        df[Columns.DISTANCE.value] = distances.flatten() * EARTH_RADIUS_KM
+        df[Columns.DISTANCE.value] = pd.Series(distances.flatten() * EARTH_RADIUS_KM, dtype=float)
 
         if df.empty:
             log.warning(
-                f"No weather stations were found for coordinate " f"{latitude}°N and {longitude}°E and number {rank}"
+                f"No weather stations_result were found for coordinate "
+                f"{latitude}°N and {longitude}°E and number {rank}"
             )
 
         return StationsResult(self, df.reset_index(drop=True))
@@ -648,13 +719,13 @@ class ScalarRequestCore(Core):
     ) -> StationsResult:
         """
         Wrapper for get_nearby_stations_by_distance using the given parameter set.
-        Returns nearest stations defined by distance (km).
+        Returns nearest stations_result defined by distance (km).
 
         :param latitude: latitude in degrees
         :param longitude: longitude in degrees
-        :param distance: distance (km) for which stations will be selected
+        :param distance: distance (km) for which stations_result will be selected
         :param unit: unit string for conversion
-        :return: pandas.DataFrame with station information for the selected stations
+        :return: pandas.DataFrame with station information for the selected stations_result
         """
         distance = float(distance)
 
@@ -673,7 +744,7 @@ class ScalarRequestCore(Core):
 
         if df.empty:
             log.warning(
-                f"No weather stations were found for coordinate "
+                f"No weather stations_result were found for coordinate "
                 f"{latitude}°N and {longitude}°E and distance {distance_in_km}km"
             )
 
@@ -681,13 +752,13 @@ class ScalarRequestCore(Core):
 
     def filter_by_bbox(self, left: float, bottom: float, right: float, top: float) -> StationsResult:
         """
-        Method to filter stations by bounding box.
+        Method to filter stations_result by bounding box.
 
         :param bottom: bottom latitude as float
         :param left: left longitude as float
         :param top: top latitude as float
         :param right: right longitude as float
-        :return: df with stations in bounding box
+        :return: df with stations_result in bounding box
         """
         left, bottom, right, top = float(left), float(bottom), float(right), float(top)
 
@@ -702,14 +773,15 @@ class ScalarRequestCore(Core):
 
         df = self.all().df
 
-        df = df[
+        df = df.loc[
             df[Columns.LATITUDE.value].apply(lambda x: x in lat_interval)
-            & df[Columns.LONGITUDE.value].apply(lambda x: x in lon_interval)
+            & df[Columns.LONGITUDE.value].apply(lambda x: x in lon_interval),
+            :,
         ]
 
         return StationsResult(stations=self, df=df.reset_index(drop=True))
 
-    def filter_by_sql(self, sql: str) -> pd.DataFrame:
+    def filter_by_sql(self, sql: str) -> StationsResult:
         """
 
         :param sql:
@@ -719,9 +791,9 @@ class ScalarRequestCore(Core):
 
         df = self.all().df
 
-        df = duckdb.query_df(df, "data", sql).df()
+        df: pd.DataFrame = duckdb.query_df(df, "data", sql).df()
 
-        df[Columns.FROM_DATE.value] = df[Columns.FROM_DATE.value].dt.tz_localize(self.tz)
-        df[Columns.TO_DATE.value] = df[Columns.TO_DATE.value].dt.tz_localize(self.tz)
+        df.loc[:, Columns.FROM_DATE.value] = df.loc[:, Columns.FROM_DATE.value].dt.tz_localize(self.tz)
+        df.loc[:, Columns.TO_DATE.value] = df.loc[:, Columns.TO_DATE.value].dt.tz_localize(self.tz)
 
         return StationsResult(stations=self, df=df.reset_index(drop=True))
