@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2018-2021, earthobservations developers.
+# Copyright (C) 2018-2021, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
 import datetime
 import logging
@@ -150,7 +150,7 @@ class ScalarValuesCore(metaclass=ABCMeta):
         if self.sr.resolution == Resolution.MONTHLY:
             end_date += pd.Timedelta(days=31)
         elif self.sr.resolution == Resolution.ANNUAL:
-            end_date += pd.Timedelta(year=366)
+            end_date += pd.Timedelta(days=366)
 
         if self._data_tz == Timezone.DYNAMIC:
             timezone_ = self._get_timezone_from_station(station_id)
@@ -240,15 +240,15 @@ class ScalarValuesCore(metaclass=ABCMeta):
 
         conversion_factors = self._create_conversion_factors(dataset)
 
-        if self.sr._has_tidy_data:
+        if self.sr.tidy:
             data = []
-            for par, group in df.groupby(Columns.PARAMETER.value):
+            for par, group in df.groupby(Columns.PARAMETER.value, sort=False):
                 group[Columns.VALUE.value] = _convert_values_to_si(group[Columns.VALUE.value], par)
                 data.append(group)
             try:
                 return pd.concat(data)
             except ValueError:
-                return pd.DataFrame()
+                return pd.DataFrame()  # columns=self._meta_fields
 
         return df.apply(_convert_values_to_si, axis=0)
 
@@ -321,11 +321,10 @@ class ScalarValuesCore(metaclass=ABCMeta):
         resolution = self.sr.resolution
 
         # if parameter is a whole dataset, take every parameter from the dataset instead
-        if parameter == dataset:
-            if self.sr._unique_dataset or not self.sr._has_datasets:
-                parameter = [*parameter_base[resolution.name]]
-            else:
-                parameter = [*parameter_base[resolution.name][dataset.name]]
+        if self.sr._unique_dataset or not self.sr._has_datasets:
+            parameter = [*parameter_base[resolution.name]]
+        else:
+            parameter = [*parameter_base[resolution.name][dataset.name]]
 
         if self.sr._has_tidy_data:
             if not self.sr.start_date:
@@ -421,14 +420,9 @@ class ScalarValuesCore(metaclass=ABCMeta):
         """
         columns = self._meta_fields.copy()
         columns.extend(df.columns.difference(columns, sort=False))
-
-        df = df.reindex(columns=columns)
-
-        # ensure meta columns content
-        df.loc[:, Columns.STATION_ID.value] = station_id
-        df.loc[:, Columns.DATASET.value] = dataset.name.lower()
-
-        return df
+        df[Columns.STATION_ID.value] = station_id
+        df[Columns.DATASET.value] = dataset.name.lower()
+        return df.reindex(columns=columns)
 
     def query(self) -> Generator[ValuesResult, None, None]:
         """
@@ -466,13 +460,13 @@ class ScalarValuesCore(metaclass=ABCMeta):
                 #  however we later again coerce when concatenating DataFrames
                 parameter_df = self._coerce_parameter_types(parameter_df)
 
-                if self.sr.si_units:
-                    parameter_df = self.convert_values_to_si(parameter_df, dataset)
-
                 if self.sr.tidy:
                     if not self.sr._has_tidy_data:
                         parameter_df = self.tidy_up_df(parameter_df, dataset)
 
+                    if Columns.PARAMETER.value not in parameter_df:
+                        parameter_df[Columns.PARAMETER.value] = parameter.value.lower()
+
                     if parameter != dataset:
                         parameter_df = parameter_df.loc[
                             parameter_df[Columns.PARAMETER.value] == parameter.value.lower(), :
@@ -480,15 +474,22 @@ class ScalarValuesCore(metaclass=ABCMeta):
                 elif self.sr._has_tidy_data:
                     parameter_df = self.tabulate_df(parameter_df)
 
+                if not parameter_df.empty and self.sr.si_units:
+                    parameter_df = self.convert_values_to_si(parameter_df, dataset)
+
                 # Skip date fields in tidy format, no further check required as still
                 # "normal" parameters should be available
                 if self.sr.tidy and self._date_parameters:
                     parameter_df = parameter_df.loc[
-                        ~parameter_df[Columns.PARAMETER.value].isin(self._date_parameters), :
+                        ~parameter_df.loc[:, Columns.PARAMETER.value].isin(self._date_parameters), :
                     ]
                     if parameter_df.empty:
                         continue
 
+                # ensure meta columns content
+                parameter_df[Columns.STATION_ID.value] = station_id
+                parameter_df[Columns.DATASET.value] = dataset.name.lower()
+
                 parameter_df = self._coerce_date_fields(parameter_df, station_id)
 
                 # Merge on full date range if values are found to ensure result
@@ -508,6 +509,18 @@ class ScalarValuesCore(metaclass=ABCMeta):
             except ValueError:
                 station_df = pd.DataFrame()
 
+            if self.sr.skip_empty:
+                ap = self._get_actual_percentage(station_df)
+
+                if ap < self.sr.skip_threshold:
+                    log.info(
+                        f"station {station_id} is skipped as percentage of actual values ({ap}) is below threshold ({self.sr.skip_threshold})."
+                    )
+                    continue
+
+            if self.sr.dropna:
+                station_df = station_df.loc[station_df.value.notna(), :].reset_index(drop=True)
+
             station_df = self._coerce_meta_fields(station_df)
 
             # Filter for dates range if start_date and end_date are defined
@@ -781,3 +794,8 @@ class ScalarValuesCore(metaclass=ABCMeta):
             parameter.value: parameter.name.lower()
             for parameter in self.sr.stations._parameter_base[self.sr.stations._dataset_accessor]
         }
+
+    @staticmethod
+    def _get_actual_percentage(df: pd.DataFrame) -> float:
+        """Calculate percentage of actual values"""
+        return df.groupby("parameter").apply(lambda x: x.value.dropna().size / x.value.size).min()
